/**
 * This chart bean supports only BAR Graphs
 *
 * @author A Bharath Reddy.
 */
import Acme.JPM.Encoders.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class ChartBean{
	private Vector values;
	private Vector titles;
	private int gifWidth;
	private int gifHeight;
	private boolean horizontal;
	private int scaleFactor;
	private boolean dynamicColors;
	private Color colors[] = {
						Color.magenta,
						Color.cyan,
						Color.blue,
						Color.red,
						Color.orange,
						Color.yellow,
						Color.green,
						Color.pink,
						Color.black,
						Color.lightGray,
						Color.darkGray,
						Color.gray
				   };

	public ChartBean(){
	}

	/**
	 * This method basically sends the The Barchart Image to the OutputStream
	 * of the servlet.
	 *
	 *
	 * @param values A vector of values against which the graph is plotted
	 * @param titles the titles to be displayed for the bar graph.
	 * @param gifWt The width of the Gif Image (100+yourwidth)
	 * @param gifHt the height of the Gif Image
	 * @param sFactor How many Cells should the bar graph occupy
	 * @param isHorizontal true speficies that the graph generated should be Horizontal
	 * @param dynamicColors if specified true, then random colors are picked
	 * @param response The javax.servlet.http.HttpServletResponse
	 */
	public synchronized void drawChart(Vector values,
						   Vector titles,
						   int gifWt,
						   int gifHt,
						   int sFactor,
						   boolean isHorizontal,
						   boolean dynamicColors,
						   HttpServletResponse response){
		this.values = values;
		this.titles = titles;
		this.gifWidth = gifWt;
		this.gifHeight = gifHt;
		this.scaleFactor = sFactor;
		this.horizontal = isHorizontal;
		this.dynamicColors = dynamicColors;
		response.setContentType("image/gif");
		drawBarChart(response);
	}

	private void drawBarChart(HttpServletResponse response){
		Frame container = new Frame();
		try{
			container.addNotify();
			Image img = null;
			img = container.createImage(gifWidth,gifHeight+20);
			Graphics g = img.getGraphics();
			double dob = getMaxValue(values);
			int cellWidth  = getRoundedValue(dob / scaleFactor);
			int maxAxisLength = cellWidth * scaleFactor;
			if( horizontal ){
				int cw = (gifWidth - 100) / scaleFactor;
				for(int i=0 ; i<scaleFactor ; i++){
					g.drawLine(100 + (cw * i),0, 100 + (cw * i), gifHeight);
					g.drawString((cellWidth * i)+"",95 + (cw * i),gifHeight+12);
				}
				int numOfBars = values.size();
				int barThickness = (gifHeight / numOfBars)-3;
				int maxColor = colors.length;
				int startColor = 0;
				for(int i=0 ; i<values.size() ; i++ ){
					double val = ((Double)values.elementAt(i)).doubleValue();
					double length = ( val * cw ) / cellWidth;
					int barLength = (int)Math.round(length);
					if( dynamicColors ){
						g.setColor(getRandomColor());
					}else{
						if( startColor == maxColor ){
							startColor = 0;
						}
						g.setColor(colors[startColor]);
						startColor++;
					}
					g.fill3DRect(100 , (gifHeight - (barThickness * (i+1)) - (3*(i+1))) , barLength, barThickness,true);
					g.setColor(Color.black);
					String str = (String)titles.elementAt(i);
					int y = (gifHeight - (barThickness * (i+1)) - (3*(i+1)));
					FontMetrics fm = g.getFontMetrics(g.getFont());
					int y1 = fm.getMaxDescent() - fm.getMaxAscent() - (barThickness/2);
					g.drawString(str,1,y-y1);
				}
			}else{
				int cw = gifHeight / scaleFactor;
				int numOfBars = values.size();
				int barThickness = ((gifWidth-100) / numOfBars)-3;
				for(int i=0 ; i<scaleFactor ; i++){
					g.drawLine(100,(cw * i),gifWidth, (cw * i));
					String str1 = (cellWidth * i)+"";
					FontMetrics fm = g.getFontMetrics(g.getFont());
					int widt = fm.stringWidth(str1);
					int xPoint = 96 - widt;
					g.drawString(str1 , xPoint , (cw * (scaleFactor - i))+3);
				}
				int maxColor = colors.length;
				int startColor = 0;
				for(int i=0 ; i<values.size() ; i++ ){
					double val = ((Double)values.elementAt(i)).doubleValue();
					double length = ( val * cw ) / cellWidth;
					int barLength = (int)Math.round(length);
					if( dynamicColors ){
						g.setColor(getRandomColor());
					}else{
						if( startColor == maxColor ){
							startColor = 0;
						}
						g.setColor(colors[startColor]);
						startColor++;
					}
					g.fill3DRect( (gifWidth - (barThickness * (i+1)) - (3*(i+1))) , gifHeight - barLength, barThickness , barLength ,true);
					g.setColor(Color.black);
					String str = (String)titles.elementAt(i);
					int y = (gifWidth - (barThickness * (i+1)) - (3*(i+1)));
					if( str.length() > 2 ){
						str = str.substring(0,2);
					}
					g.drawString(str,y + (barThickness/2),gifHeight+15);
				}
			}
			g.setColor(Color.black);
			g.drawRect(100,0,gifWidth-101,gifHeight);
			ServletOutputStream out = response.getOutputStream();
			GifEncoder gif = new GifEncoder(img,out);
			gif.encode();
			out.flush();
		}catch(Exception ex){
			ex.printStackTrace();
		}finally{
			container.dispose();
		}
	}

	private int getRoundedValue(double dob){
		String mvl = ""+dob;
		mvl = mvl.substring(0,mvl.indexOf('.'));
		int numLen = mvl.length();
		int num = 1;
		for(int i=1 ; i < numLen ; i++ ){
			num = num * 10;
		}
		String st = mvl.charAt(0) + "";
		int tempNum = Integer.parseInt(st.trim());
		int maxAxisLength = (num * tempNum) + num;
		return maxAxisLength;
	}

	private double getMaxValue(Vector values){
		double maxValue = 0.0d;
		for(int i=0 ; i<values.size() ; i++ ){
			double temp = ((Double)values.elementAt(i)).doubleValue();
			if( maxValue < temp ){
				maxValue = temp;
			}
		}
		return maxValue;
	}

	private Color getRandomColor(){
		int r = (int)(255 * Math.random());
		int g = (int)(255 * Math.random());
		int b = (int)(255 * Math.random());
		if( r > 225 && g > 225 && b > 255 ){
			r = 0;
		}
		return (new Color(r,g,b));
	}
}